from django import forms

from .models import Plan


class PaymentMethodForm(forms.Form):

    expMonth = forms.IntegerField(min_value=1, max_value=12)
    expYear = forms.IntegerField(min_value=2015, max_value=9999)


class PlanForm(forms.Form):
    plan = forms.ModelChoiceField(queryset=Plan.objects.all())



from __future__ import unicode_literals

import datetime
import decimal
import sys

from django.test import TestCase
from django.utils import timezone

from mock import patch

from ..models import Charge, Customer, Event, EventProcessingException, Invoice, InvoiceItem, Plan, Subscription


def _str(obj):
    if sys.version_info < (3, 0):
        return str(obj).decode("utf-8")
    else:
        return str(obj)


class ModelTests(TestCase):

    def test_plan_str(self):
        p = Plan(amount=decimal.Decimal("5"), name="My Plan", interval="monthly", interval_count=1)
        self.assertTrue(p.name in _str(p))

    def test_plan_str_usd(self):
        p = Plan(amount=decimal.Decimal("5"), name="My Plan", currency="usd", interval="monthly", interval_count=1)
        self.assertTrue(u"\u0024" in _str(p))

    def test_plan_str_jpy(self):
        p = Plan(amount=decimal.Decimal("5"), name="My Plan", currency="jpy", interval="monthly", interval_count=1)
        self.assertTrue(u"\u00a5" in _str(p))

    def test_event_processing_exception_str(self):
        e = EventProcessingException(data="hello", message="hi there", traceback="fake")
        self.assertTrue("Event=" in str(e))

    def test_event_str(self):
        e = Event(kind="customer.deleted", webhook_message={})
        self.assertTrue("customer.deleted" in str(e))

    def test_customer_str(self):
        e = Customer()
        self.assertTrue("None" in str(e))

    def test_plan_display_invoiceitem(self):
        p = Plan(amount=decimal.Decimal("5"), name="My Plan", interval="monthly", interval_count=1)
        p.save()
        i = InvoiceItem(plan=p)
        self.assertEquals(i.plan_display(), "My Plan")

    def test_model_table_name(self):
        self.assertEquals(Customer()._meta.db_table, "pinax_stripe_customer")

    def test_event_message(self):
        event = Event(validated_message={"foo": 1})
        self.assertEquals(event.validated_message, event.message)

    def test_invoice_status(self):
        self.assertEquals(Invoice(paid=True).status, "Paid")

    def test_invoice_status_not_paid(self):
        self.assertEquals(Invoice(paid=False).status, "Open")

    def test_subscription_total_amount(self):
        sub = Subscription(plan=Plan(name="Pro Plan", amount=decimal.Decimal("100")), quantity=2)
        self.assertEquals(sub.total_amount, decimal.Decimal("200"))

    def test_subscription_plan_display(self):
        sub = Subscription(plan=Plan(name="Pro Plan"))
        self.assertEquals(sub.plan_display(), "Pro Plan")

    def test_subscription_status_display(self):
        sub = Subscription(status="overly_active")
        self.assertEquals(sub.status_display(), "Overly Active")

    def test_subscription_delete(self):
        plan = Plan.objects.create(stripe_id="pro2", amount=decimal.Decimal("100"), interval="monthly", interval_count=1)
        customer = Customer.objects.create(stripe_id="foo")
        sub = Subscription.objects.create(customer=customer, status="trialing", start=timezone.now(), plan=plan, quantity=1, cancel_at_period_end=True, current_period_end=(timezone.now() - datetime.timedelta(days=2)))
        sub.delete()
        self.assertIsNone(sub.status)
        self.assertEquals(sub.quantity, 0)
        self.assertEquals(sub.amount, 0)


class StripeObjectTests(TestCase):

    @patch("stripe.Charge.retrieve")
    def test_stripe_charge(self, RetrieveMock):
        Charge().stripe_charge
        self.assertTrue(RetrieveMock.called)

    @patch("stripe.Customer.retrieve")
    def test_stripe_customer(self, RetrieveMock):
        Customer().stripe_customer
        self.assertTrue(RetrieveMock.called)

    @patch("stripe.Invoice.retrieve")
    def test_stripe_invoice(self, RetrieveMock):
        Invoice().stripe_invoice
        self.assertTrue(RetrieveMock.called)

    @patch("stripe.Customer.retrieve")
    def test_stripe_subscription(self, RetrieveMock):
        Subscription(customer=Customer(stripe_id="foo")).stripe_subscription
        self.assertTrue(RetrieveMock().subscriptions.retrieve.called)



import decimal

from django.core import management
from django.test import TestCase

from django.contrib.auth import get_user_model

from mock import patch

from ..models import Customer, Plan


class CommandTests(TestCase):

    def setUp(self):
        User = get_user_model()
        self.user = User.objects.create_user(username="patrick")

    @patch("stripe.Customer.retrieve")
    @patch("stripe.Customer.create")
    def test_init_customer_creates_customer(self, CreateMock, RetrieveMock):
        CreateMock.return_value = dict(
            account_balance=0,
            delinquent=False,
            default_source="card_178Zqj2eZvKYlo2Cr2fUZZz7",
            currency="usd",
            id="cus_XXXXX",
            sources=dict(data=[]),
            subscriptions=dict(data=[]),
        )
        management.call_command("init_customers")
        customer = Customer.objects.get(user=self.user)
        self.assertEquals(customer.stripe_id, "cus_XXXXX")

    @patch("stripe.Plan.all")
    @patch("stripe.Plan.auto_paging_iter", create=True, side_effect=AttributeError)
    def test_plans_create_deprecated(self, PlanAutoPagerMock, PlanAllMock):
        PlanAllMock().data = [{
            "id": "entry-monthly",
            "amount": 954,
            "interval": "monthly",
            "interval_count": 1,
            "currency": None,
            "statement_descriptor": None,
            "trial_period_days": None,
            "name": "Pro"
        }]
        management.call_command("sync_plans")
        self.assertEquals(Plan.objects.count(), 1)
        self.assertEquals(Plan.objects.all()[0].stripe_id, "entry-monthly")
        self.assertEquals(Plan.objects.all()[0].amount, decimal.Decimal("9.54"))

    @patch("stripe.Plan.auto_paging_iter", create=True)
    def test_plans_create(self, PlanAutoPagerMock):
        PlanAutoPagerMock.return_value = [{
            "id": "entry-monthly",
            "amount": 954,
            "interval": "monthly",
            "interval_count": 1,
            "currency": None,
            "statement_descriptor": None,
            "trial_period_days": None,
            "name": "Pro"
        }]
        management.call_command("sync_plans")
        self.assertEquals(Plan.objects.count(), 1)
        self.assertEquals(Plan.objects.all()[0].stripe_id, "entry-monthly")
        self.assertEquals(Plan.objects.all()[0].amount, decimal.Decimal("9.54"))

    @patch("stripe.Customer.retrieve")
    @patch("pinax.stripe.actions.customers.sync_customer")
    @patch("pinax.stripe.actions.invoices.sync_invoices_for_customer")
    @patch("pinax.stripe.actions.charges.sync_charges_for_customer")
    def test_sync_customers(self, SyncChargesMock, SyncInvoicesMock, SyncMock, RetrieveMock):
        user2 = get_user_model().objects.create_user(username="thomas")
        get_user_model().objects.create_user(username="altman")
        Customer.objects.create(stripe_id="cus_XXXXX", user=self.user)
        Customer.objects.create(stripe_id="cus_YYYYY", user=user2)
        management.call_command("sync_customers")
        self.assertEqual(SyncChargesMock.call_count, 2)
        self.assertEqual(SyncInvoicesMock.call_count, 2)
        self.assertEqual(SyncMock.call_count, 2)



import json

from django.http import HttpResponse
from django.shortcuts import redirect
from django.utils.decorators import method_decorator
from django.utils.encoding import smart_str
from django.views.generic import TemplateView, DetailView, View, FormView, ListView
from django.views.generic.edit import FormMixin
from django.views.decorators.csrf import csrf_exempt

import stripe

from .actions import events, exceptions, customers, subscriptions, sources
from .forms import PlanForm, PaymentMethodForm
from .mixins import LoginRequiredMixin, CustomerMixin, PaymentsContextMixin
from .models import Invoice, Card, Subscription


class InvoiceListView(LoginRequiredMixin, CustomerMixin, ListView):
    model = Invoice
    context_object_name = "invoice_list"
    template_name = "pinax/stripe/invoice_list.html"

    def get_queryset(self):
        return super(InvoiceListView, self).get_queryset().order_by("date")


class PaymentMethodListView(LoginRequiredMixin, CustomerMixin, ListView):
    model = Card
    context_object_name = "payment_method_list"
    template_name = "pinax/stripe/paymentmethod_list.html"

    def get_queryset(self):
        return super(PaymentMethodListView, self).get_queryset().order_by("created_at")


class PaymentMethodCreateView(LoginRequiredMixin, CustomerMixin, PaymentsContextMixin, TemplateView):
    model = Card
    template_name = "pinax/stripe/paymentmethod_create.html"

    def create_card(self, stripe_token):
        sources.create_card(self.customer, token=stripe_token)

    def post(self, request, *args, **kwargs):
        try:
            self.create_card(request.POST.get("stripeToken"))
            return redirect("pinax_stripe_payment_method_list")
        except stripe.CardError as e:
            return self.render_to_response(self.get_context_data(errors=smart_str(e)))


class PaymentMethodDeleteView(LoginRequiredMixin, CustomerMixin, DetailView):
    model = Card
    template_name = "pinax/stripe/paymentmethod_delete.html"

    def delete_card(self, stripe_id):
        sources.delete_card(self.customer, stripe_id)

    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        try:
            self.delete_card(self.object.stripe_id)
            return redirect("pinax_stripe_payment_method_list")
        except stripe.CardError as e:
            return self.render_to_response(self.get_context_data(errors=smart_str(e)))


class PaymentMethodUpdateView(LoginRequiredMixin, CustomerMixin, PaymentsContextMixin, FormMixin, DetailView):
    model = Card
    form_class = PaymentMethodForm
    template_name = "pinax/stripe/paymentmethod_update.html"

    def update_card(self, exp_month, exp_year):
        sources.update_card(self.customer, self.object.stripe_id, exp_month=exp_month, exp_year=exp_year)

    def form_valid(self, form):
        try:
            self.update_card(form.cleaned_data["expMonth"], form.cleaned_data["expYear"])
            return redirect("pinax_stripe_payment_method_list")
        except stripe.CardError as e:
            return self.render_to_response(self.get_context_data(errors=smart_str(e)))

    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        form = self.get_form(form_class=self.form_class)
        if form.is_valid():
            return self.form_valid(form)
        else:
            return self.form_invalid(form)


class SubscriptionListView(LoginRequiredMixin, CustomerMixin, ListView):
    model = Subscription
    context_object_name = "subscription_list"
    template_name = "pinax/stripe/subscription_list.html"

    def get_queryset(self):
        return super(SubscriptionListView, self).get_queryset().order_by("created_at")


class SubscriptionCreateView(LoginRequiredMixin, PaymentsContextMixin, CustomerMixin, FormView):
    template_name = "pinax/stripe/subscription_create.html"
    form_class = PlanForm

    def set_customer(self):
        if self.customer is None:
            self._customer = customers.create(self.request.user)

    def subscribe(self, customer, plan, token):
        subscriptions.create(customer, plan, token=token)

    def form_valid(self, form):
        self.set_customer()
        try:
            self.subscribe(self.customer, plan=form.cleaned_data["plan"], token=self.request.POST.get("stripeToken"))
            return redirect("pinax_stripe_subscription_list")
        except stripe.StripeError as e:
            return self.render_to_response(self.get_context_data(form=form, errors=smart_str(e)))


class SubscriptionDeleteView(LoginRequiredMixin, CustomerMixin, DetailView):
    model = Subscription
    template_name = "pinax/stripe/subscription_delete.html"

    def cancel(self):
        subscriptions.cancel(self.object)

    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        try:
            self.cancel()
            return redirect("pinax_stripe_subscription_list")
        except stripe.StripeError as e:
            return self.render_to_response(self.get_context_data(errors=smart_str(e)))


class SubscriptionUpdateView(LoginRequiredMixin, CustomerMixin, FormMixin, DetailView):
    model = Subscription
    form_class = PlanForm
    template_name = "pinax/stripe/subscription_update.html"

    @property
    def current_plan(self):
        if not hasattr(self, "_current_plan"):
            self._current_plan = self.object.plan
        return self._current_plan

    def get_context_data(self, **kwargs):
        context = super(SubscriptionUpdateView, self).get_context_data(**kwargs)
        context.update({
            "form": self.get_form(form_class=self.form_class)
        })
        return context

    def update_subscription(self, plan_id):
        subscriptions.update(self.object, plan_id)

    def get_initial(self):
        initial = super(SubscriptionUpdateView, self).get_initial()
        initial.update({
            "plan": self.current_plan
        })
        return initial

    def form_valid(self, form):
        try:
            self.update_subscription(form.cleaned_data["plan"])
            return redirect("pinax_stripe_subscription_list")
        except stripe.StripeError as e:
            return self.render_to_response(self.get_context_data(form=form, errors=smart_str(e)))

    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        form = self.get_form(form_class=self.form_class)
        if form.is_valid():
            return self.form_valid(form)
        else:
            return self.form_invalid(form)


class Webhook(View):

    @method_decorator(csrf_exempt)
    def dispatch(self, *args, **kwargs):
        return super(Webhook, self).dispatch(*args, **kwargs)

    def extract_json(self):
        data = json.loads(smart_str(self.request.body))
        return data

    def post(self, request, *args, **kwargs):
        data = self.extract_json()
        if events.dupe_event_exists(data["id"]):
            exceptions.log_exception(data, "Duplicate event record")
        else:
            events.add_event(
                stripe_id=data["id"],
                kind=data["type"],
                livemode=data["livemode"],
                message=data
            )
        return HttpResponse()



from django.core.urlresolvers import reverse
from django.test import TestCase
from django.utils import timezone

from django.contrib.auth import get_user_model

import stripe

from mock import patch

from ..models import Card, Customer, Invoice, Plan, Subscription
from ..views import PaymentMethodCreateView


class PaymentsContextMixinTests(TestCase):

    def test_payments_context_mixin_get_context_data(self):
        data = PaymentMethodCreateView().get_context_data()
        self.assertTrue("PINAX_STRIPE_PUBLIC_KEY" in data)


class InvoiceListViewTests(TestCase):

    def setUp(self):
        self.password = "eldarion"
        self.user = get_user_model().objects.create_user(
            username="patrick",
            password=self.password
        )
        self.user.save()
        customer = Customer.objects.create(
            stripe_id="cus_1",
            user=self.user
        )
        Invoice.objects.create(
            stripe_id="inv_001",
            customer=customer,
            amount_due=100,
            period_end=timezone.now(),
            period_start=timezone.now(),
            subtotal=100,
            total=100,
            date=timezone.now()
        )
        Invoice.objects.create(
            stripe_id="inv_002",
            customer=customer,
            amount_due=50,
            period_end=timezone.now(),
            period_start=timezone.now(),
            subtotal=50,
            total=50,
            date=timezone.now()
        )

    def test_context(self):
        self.client.login(username=self.user.username, password=self.password)
        response = self.client.get(
            reverse("pinax_stripe_invoice_list")
        )
        self.assertTrue("invoice_list" in response.context_data)
        self.assertEquals(response.context_data["invoice_list"].count(), 2)
        self.assertEquals(response.context_data["invoice_list"][0].total, 100)
        self.assertEquals(response.context_data["invoice_list"][1].total, 50)


class PaymentMethodListViewTests(TestCase):

    def setUp(self):
        self.password = "eldarion"
        self.user = get_user_model().objects.create_user(
            username="patrick",
            password=self.password
        )
        self.user.save()
        customer = Customer.objects.create(
            stripe_id="cus_1",
            user=self.user
        )
        Card.objects.create(
            stripe_id="card_001",
            customer=customer,
            address_line_1_check="nothing",
            address_zip_check="nothing",
            country="US",
            cvc_check="passed",
            exp_month=1,
            exp_year=2020,
            funding="yes",
            fingerprint="abc"
        )

    def test_context(self):
        self.client.login(username=self.user.username, password=self.password)
        response = self.client.get(
            reverse("pinax_stripe_payment_method_list")
        )
        self.assertTrue("payment_method_list" in response.context_data)
        self.assertEquals(response.context_data["payment_method_list"].count(), 1)
        self.assertEquals(response.context_data["payment_method_list"][0].stripe_id, "card_001")


class PaymentMethodCreateViewTests(TestCase):

    def setUp(self):
        self.password = "eldarion"
        self.user = get_user_model().objects.create_user(
            username="patrick",
            password=self.password
        )
        self.user.save()
        Customer.objects.create(
            stripe_id="cus_1",
            user=self.user
        )

    @patch("pinax.stripe.actions.sources.create_card")
    def test_post(self, CreateMock):
        self.client.login(username=self.user.username, password=self.password)
        response = self.client.post(
            reverse("pinax_stripe_payment_method_create"),
            {}
        )
        self.assertEquals(response.status_code, 302)
        self.assertRedirects(response, reverse("pinax_stripe_payment_method_list"))

    @patch("pinax.stripe.actions.sources.create_card")
    def test_post_on_error(self, CreateMock):
        CreateMock.side_effect = stripe.CardError("Bad card", "Param", "CODE")
        self.client.login(username=self.user.username, password=self.password)
        response = self.client.post(
            reverse("pinax_stripe_payment_method_create"),
            {}
        )
        self.assertEquals(response.status_code, 200)
        self.assertTrue("errors" in response.context_data)


class PaymentMethodDeleteViewTests(TestCase):

    def setUp(self):
        self.password = "eldarion"
        self.user = get_user_model().objects.create_user(
            username="patrick",
            password=self.password
        )
        self.user.save()
        customer = Customer.objects.create(
            stripe_id="cus_1",
            user=self.user
        )
        self.card = Card.objects.create(
            stripe_id="card_001",
            customer=customer,
            address_line_1_check="nothing",
            address_zip_check="nothing",
            country="US",
            cvc_check="passed",
            exp_month=1,
            exp_year=2020,
            funding="yes",
            fingerprint="abc"
        )

    @patch("pinax.stripe.actions.sources.delete_card")
    def test_post(self, CreateMock):
        self.client.login(username=self.user.username, password=self.password)
        response = self.client.post(
            reverse("pinax_stripe_payment_method_delete", args=[self.card.pk]),
            {}
        )
        self.assertEquals(response.status_code, 302)
        self.assertRedirects(response, reverse("pinax_stripe_payment_method_list"))

    @patch("pinax.stripe.actions.sources.delete_card")
    def test_post_on_error(self, CreateMock):
        CreateMock.side_effect = stripe.CardError("Bad card", "Param", "CODE")
        self.client.login(username=self.user.username, password=self.password)
        response = self.client.post(
            reverse("pinax_stripe_payment_method_delete", args=[self.card.pk]),
            {}
        )
        self.assertEquals(response.status_code, 200)
        self.assertTrue("errors" in response.context_data)


class PaymentMethodUpdateViewTests(TestCase):

    def setUp(self):
        self.password = "eldarion"
        self.user = get_user_model().objects.create_user(
            username="patrick",
            password=self.password
        )
        self.user.save()
        customer = Customer.objects.create(
            stripe_id="cus_1",
            user=self.user
        )
        self.card = Card.objects.create(
            stripe_id="card_001",
            customer=customer,
            address_line_1_check="nothing",
            address_zip_check="nothing",
            country="US",
            cvc_check="passed",
            exp_month=1,
            exp_year=2020,
            funding="yes",
            fingerprint="abc"
        )

    @patch("pinax.stripe.actions.sources.update_card")
    def test_post(self, CreateMock):
        self.client.login(username=self.user.username, password=self.password)
        response = self.client.post(
            reverse("pinax_stripe_payment_method_update", args=[self.card.pk]),
            {
                "expMonth": 1,
                "expYear": 2018
            }
        )
        self.assertEquals(response.status_code, 302)
        self.assertRedirects(response, reverse("pinax_stripe_payment_method_list"))

    @patch("pinax.stripe.actions.sources.update_card")
    def test_post_invalid_form(self, CreateMock):
        self.client.login(username=self.user.username, password=self.password)
        response = self.client.post(
            reverse("pinax_stripe_payment_method_update", args=[self.card.pk]),
            {
                "expMonth": 13,
                "expYear": 2014
            }
        )
        self.assertEquals(response.status_code, 200)
        self.assertEquals(response.context_data["form"].is_valid(), False)

    @patch("pinax.stripe.actions.sources.update_card")
    def test_post_on_error(self, CreateMock):
        CreateMock.side_effect = stripe.CardError("Bad card", "Param", "CODE")
        self.client.login(username=self.user.username, password=self.password)
        response = self.client.post(
            reverse("pinax_stripe_payment_method_update", args=[self.card.pk]),
            {
                "expMonth": 1,
                "expYear": 2018
            }
        )
        self.assertEquals(response.status_code, 200)
        self.assertTrue("errors" in response.context_data)


class SubscriptionListViewTests(TestCase):

    def setUp(self):
        self.password = "eldarion"
        self.user = get_user_model().objects.create_user(
            username="patrick",
            password=self.password
        )
        self.user.save()
        customer = Customer.objects.create(
            stripe_id="cus_1",
            user=self.user
        )
        plan = Plan.objects.create(
            amount=10,
            currency="usd",
            interval="monthly",
            interval_count=1,
            name="Pro"
        )
        Subscription.objects.create(
            stripe_id="sub_001",
            customer=customer,
            plan=plan,
            quantity=1,
            start=timezone.now(),
            status="active"
        )

    def test_context(self):
        self.client.login(username=self.user.username, password=self.password)
        response = self.client.get(
            reverse("pinax_stripe_subscription_list")
        )
        self.assertTrue("subscription_list" in response.context_data)
        self.assertEquals(response.context_data["subscription_list"].count(), 1)
        self.assertEquals(response.context_data["subscription_list"][0].stripe_id, "sub_001")


class SubscriptionCreateViewTests(TestCase):

    def setUp(self):
        self.password = "eldarion"
        self.user = get_user_model().objects.create_user(
            username="patrick",
            password=self.password
        )
        self.user.save()
        self.plan = Plan.objects.create(
            amount=10,
            currency="usd",
            interval="monthly",
            interval_count=1,
            name="Pro"
        )

    @patch("pinax.stripe.actions.subscriptions.create")
    def test_post(self, CreateMock):
        Customer.objects.create(
            stripe_id="cus_1",
            user=self.user
        )
        self.client.login(username=self.user.username, password=self.password)
        response = self.client.post(
            reverse("pinax_stripe_subscription_create"),
            {
                "plan": self.plan.id
            }
        )
        self.assertEquals(response.status_code, 302)
        self.assertRedirects(response, reverse("pinax_stripe_subscription_list"))

    @patch("pinax.stripe.actions.customers.create")
    @patch("pinax.stripe.actions.subscriptions.create")
    def test_post_no_prior_customer(self, CreateMock, CustomerCreateMock):
        self.client.login(username=self.user.username, password=self.password)
        response = self.client.post(
            reverse("pinax_stripe_subscription_create"),
            {
                "plan": self.plan.id
            }
        )
        self.assertEquals(response.status_code, 302)
        self.assertRedirects(response, reverse("pinax_stripe_subscription_list"))
        self.assertTrue(CustomerCreateMock.called)

    @patch("pinax.stripe.actions.sources.create_card")
    def test_post_on_error(self, CreateMock):
        Customer.objects.create(
            stripe_id="cus_1",
            user=self.user
        )
        CreateMock.side_effect = stripe.StripeError("Bad Mojo", "Param", "CODE")
        self.client.login(username=self.user.username, password=self.password)
        response = self.client.post(
            reverse("pinax_stripe_subscription_create"),
            {
                "plan": self.plan.id
            }
        )
        self.assertEquals(response.status_code, 200)
        self.assertTrue("errors" in response.context_data)


class SubscriptionDeleteViewTests(TestCase):

    def setUp(self):
        self.password = "eldarion"
        self.user = get_user_model().objects.create_user(
            username="patrick",
            password=self.password
        )
        self.user.save()
        customer = Customer.objects.create(
            stripe_id="cus_1",
            user=self.user
        )
        plan = Plan.objects.create(
            amount=10,
            currency="usd",
            interval="monthly",
            interval_count=1,
            name="Pro"
        )
        self.subscription = Subscription.objects.create(
            stripe_id="sub_001",
            customer=customer,
            plan=plan,
            quantity=1,
            start=timezone.now(),
            status="active"
        )

    @patch("pinax.stripe.actions.subscriptions.cancel")
    def test_post(self, CancelMock):
        self.client.login(username=self.user.username, password=self.password)
        response = self.client.post(
            reverse("pinax_stripe_subscription_delete", args=[self.subscription.pk]),
            {}
        )
        self.assertEquals(response.status_code, 302)
        self.assertRedirects(response, reverse("pinax_stripe_subscription_list"))

    @patch("pinax.stripe.actions.subscriptions.cancel")
    def test_post_on_error(self, CancelMock):
        CancelMock.side_effect = stripe.StripeError("Bad Foo", "Param", "CODE")
        self.client.login(username=self.user.username, password=self.password)
        response = self.client.post(
            reverse("pinax_stripe_subscription_delete", args=[self.subscription.pk]),
            {}
        )
        self.assertEquals(response.status_code, 200)
        self.assertTrue("errors" in response.context_data)


class SubscriptionUpdateViewTests(TestCase):

    def setUp(self):
        self.password = "eldarion"
        self.user = get_user_model().objects.create_user(
            username="patrick",
            password=self.password
        )
        self.user.save()
        customer = Customer.objects.create(
            stripe_id="cus_1",
            user=self.user
        )
        plan = Plan.objects.create(
            amount=10,
            currency="usd",
            interval="monthly",
            interval_count=1,
            name="Pro"
        )
        self.subscription = Subscription.objects.create(
            stripe_id="sub_001",
            customer=customer,
            plan=plan,
            quantity=1,
            start=timezone.now(),
            status="active"
        )

    def test_get(self):
        self.client.login(username=self.user.username, password=self.password)
        response = self.client.get(
            reverse("pinax_stripe_subscription_update", args=[self.subscription.pk])
        )
        self.assertEquals(response.status_code, 200)
        self.assertTrue("form" in response.context_data)
        self.assertTrue(response.context_data["form"].initial["plan"], self.subscription.plan)

    @patch("pinax.stripe.actions.subscriptions.update")
    def test_post(self, UpdateMock):
        self.client.login(username=self.user.username, password=self.password)
        response = self.client.post(
            reverse("pinax_stripe_subscription_update", args=[self.subscription.pk]),
            {
                "plan": self.subscription.plan.id
            }
        )
        self.assertEquals(response.status_code, 302)
        self.assertRedirects(response, reverse("pinax_stripe_subscription_list"))

    @patch("pinax.stripe.actions.subscriptions.update")
    def test_post_invalid(self, UpdateMock):
        self.client.login(username=self.user.username, password=self.password)
        response = self.client.post(
            reverse("pinax_stripe_subscription_update", args=[self.subscription.pk]),
            {
                "plan": "not a real plan"
            }
        )
        self.assertEquals(response.status_code, 200)
        self.assertTrue(len(response.context_data["form"].errors) > 0)

    @patch("pinax.stripe.actions.subscriptions.update")
    def test_post_on_error(self, UpdateMock):
        UpdateMock.side_effect = stripe.StripeError("Bad Foo", "Param", "CODE")
        self.client.login(username=self.user.username, password=self.password)
        response = self.client.post(
            reverse("pinax_stripe_subscription_update", args=[self.subscription.pk]),
            {
                "plan": self.subscription.plan.id
            }
        )
        self.assertEquals(response.status_code, 200)
        self.assertTrue("errors" in response.context_data)



import decimal


from django.conf import settings

import stripe

from .. import hooks
from .. import models
from .. import utils


def calculate_refund_amount(charge, amount=None):
    """
    Calculates the refund amount given a charge and optional amount.

    Args:
        charge: a pinax.stripe.models.Charge object
        amount: optionally, the decimal.Decimal amount you wish to refund
    """
    eligible_to_refund = charge.amount - (charge.amount_refunded or 0)
    if amount:
        return min(eligible_to_refund, amount)
    return eligible_to_refund


def capture(charge, amount=None):
    """
    Capture the payment of an existing, uncaptured, charge.

    Args:
        charge: a pinax.stripe.models.Charge object
        amount: the decimal.Decimal amount of the charge to capture
    """
    stripe_charge = charge.stripe_charge.capture(
        amount=utils.convert_amount_for_api(
            amount if amount else charge.amount,
            charge.currency
        )
    )
    sync_charge_from_stripe_data(stripe_charge)


def create(amount, customer, source=None, currency="usd", description=None, send_receipt=settings.PINAX_STRIPE_SEND_EMAIL_RECEIPTS, capture=True):
    """
    Creates a charge for the given customer.

    Args:
        amount: should be a decimal.Decimal amount
        customer: the Stripe id of the customer to charge
        source: the Stripe id of the source belonging to the customer
        currency: the currency with which to charge the amount in
        description: a description of the charge
        send_receipt: send a receipt upon successful charge
        capture: immediately capture the charge instead of doing a pre-authorization

    Returns:
        a pinax.stripe.models.Charge object
    """
    if not isinstance(amount, decimal.Decimal):
        raise ValueError(
            "You must supply a decimal value representing dollars."
        )
    stripe_charge = stripe.Charge.create(
        amount=utils.convert_amount_for_api(amount, currency),  # find the final amount
        currency=currency,
        source=source,
        customer=customer,
        description=description,
        capture=capture,
    )
    charge = sync_charge_from_stripe_data(stripe_charge)
    if send_receipt:
        hooks.hookset.send_receipt(charge)
    return charge


def sync_charges_for_customer(customer):
    """
    Populate database with all the charges for a customer.

    Args:
        customer: a pinax.stripe.models.Customer object
    """
    for charge in customer.stripe_customer.charges().data:
        sync_charge_from_stripe_data(charge)


def sync_charge_from_stripe_data(data):
    """
    Create or update the charge represented by the data from a Stripe API query.

    Args:
        data: the data representing a charge object in the Stripe API

    Returns:
        a pinax.stripe.models.Charge object
    """
    customer = models.Customer.objects.get(stripe_id=data["customer"])
    obj, _ = models.Charge.objects.get_or_create(
        customer=customer,
        stripe_id=data["id"]
    )
    obj.source = data["source"]["id"]
    obj.currency = data["currency"]
    obj.invoice = next(iter(models.Invoice.objects.filter(stripe_id=data["invoice"])), None)
    obj.amount = utils.convert_amount_for_db(data["amount"], obj.currency)
    obj.paid = data["paid"]
    obj.refunded = data["refunded"]
    obj.captured = data["captured"]
    obj.disputed = data["dispute"] is not None
    obj.charge_created = utils.convert_tstamp(data, "created")
    if data.get("description"):
        obj.description = data["description"]
    if data.get("amount_refunded"):
        obj.amount_refunded = utils.convert_amount_for_db(data["amount_refunded"], obj.currency)
    if data["refunded"]:
        obj.amount_refunded = obj.amount
    obj.save()
    return obj



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models, connection


def migrate_customers(apps, schema_editor):
    cursor = connection.cursor()
    if "payments_customer" in connection.introspection.table_names():
        cursor.execute("SELECT user_id, stripe_id, date_purged FROM payments_customer")
        Customer = apps.get_model("pinax_stripe", "Customer")
        for row in cursor.fetchall():
            Customer.objects.create(
                user_id=row[0],
                stripe_id=row[1],
                date_purged=row[2]
            )


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('pinax_stripe', '0001_initial'),
    ]

    operations = [
        migrations.RunPython(migrate_customers)
    ]



from django.conf.urls import url
from .mock_views import MockView
from ..urls import urlpatterns

urlpatterns += [
    url(
        r'^the/app/$',
        MockView.as_view(),
        name='the_app'
    ),
    url(
        r'^accounts/signup/$',
        MockView.as_view(),
        name='signup'
    ),
    url(
        r'^password/reset/confirm/(?P<token>.+)/$',
        MockView.as_view(),
        name='password_reset'
    ),
]



from django.conf.urls import url

from .views import (
    InvoiceListView,

    PaymentMethodListView,
    PaymentMethodCreateView,
    PaymentMethodDeleteView,
    PaymentMethodUpdateView,

    SubscriptionListView,
    SubscriptionCreateView,
    SubscriptionDeleteView,
    SubscriptionUpdateView,

    Webhook,
)


urlpatterns = [
    url(r"^subscriptions/$", SubscriptionListView.as_view(), name="pinax_stripe_subscription_list"),
    url(r"^subscriptions/create/$", SubscriptionCreateView.as_view(), name="pinax_stripe_subscription_create"),
    url(r"^subscriptions/(?P<pk>\d+)/delete/$", SubscriptionDeleteView.as_view(), name="pinax_stripe_subscription_delete"),
    url(r"^subscriptions/(?P<pk>\d+)/update/$", SubscriptionUpdateView.as_view(), name="pinax_stripe_subscription_update"),

    url(r"^payment-methods/$", PaymentMethodListView.as_view(), name="pinax_stripe_payment_method_list"),
    url(r"^payment-methods/create/$", PaymentMethodCreateView.as_view(), name="pinax_stripe_payment_method_create"),
    url(r"^payment-methods/(?P<pk>\d+)/delete/$", PaymentMethodDeleteView.as_view(), name="pinax_stripe_payment_method_delete"),
    url(r"^payment-methods/(?P<pk>\d+)/update/$", PaymentMethodUpdateView.as_view(), name="pinax_stripe_payment_method_update"),

    url(r"^invoices/$", InvoiceListView.as_view(), name="pinax_stripe_invoice_list"),

    url(r"^webhook/$", Webhook.as_view(), name="pinax_stripe_webhook"),
]



from django.core.urlresolvers import resolve
from django.shortcuts import redirect

from .actions import customers, subscriptions
from .conf import settings


class ActiveSubscriptionMiddleware(object):

    def process_request(self, request):
        if request.user.is_authenticated() and not request.user.is_staff:
            url_name = resolve(request.path).url_name
            if url_name not in settings.PINAX_STRIPE_SUBSCRIPTION_REQUIRED_EXCEPTION_URLS:
                customer = customers.get_customer_for_user(request.user)
                if not subscriptions.has_active_subscription(customer):
                    return redirect(
                        settings.PINAX_STRIPE_SUBSCRIPTION_REQUIRED_REDIRECT
                    )



import decimal

from django.test import TestCase

from django.contrib.auth import get_user_model

from ..hooks import DefaultHookSet
from ..models import Charge, Customer


class HooksTestCase(TestCase):

    def setUp(self):
        self.User = get_user_model()
        self.user = self.User.objects.create_user(
            username="patrick",
            email="paltman@eldarion.com"
        )
        self.customer = Customer.objects.create(
            user=self.user,
            stripe_id="cus_xxxxxxxxxxxxxxx"
        )
        self.hookset = DefaultHookSet()

    def test_adjust_subscription_quantity(self):
        new_qty = self.hookset.adjust_subscription_quantity(customer=None, plan=None, quantity=3)
        self.assertEquals(new_qty, 3)

    def test_adjust_subscription_quantity_none(self):
        new_qty = self.hookset.adjust_subscription_quantity(customer=None, plan=None, quantity=None)
        self.assertEquals(new_qty, 1)

    def test_trial_period(self):
        period = self.hookset.trial_period(self.user, "some plan")
        self.assertIsNone(period)

    def test_send_receipt(self):
        charge = Charge.objects.create(
            stripe_id="ch_XXXXXX",
            customer=self.customer,
            source="card_01",
            amount=decimal.Decimal("10.00"),
            currency="usd",
            paid=True,
            refunded=False,
            disputed=False,
            receipt_sent=False
        )
        self.hookset.send_receipt(charge)
        self.assertTrue(Charge.objects.get(pk=charge.pk).receipt_sent)

    def test_send_receipt_already_sent(self):
        charge = Charge.objects.create(
            stripe_id="ch_XXXXXX",
            customer=self.customer,
            source="card_01",
            amount=decimal.Decimal("10.00"),
            currency="usd",
            paid=True,
            refunded=False,
            disputed=False,
            receipt_sent=True
        )
        self.hookset.send_receipt(charge)
        self.assertTrue(Charge.objects.get(pk=charge.pk).receipt_sent)



import importlib

from django.conf import settings  # noqa
from django.core.exceptions import ImproperlyConfigured

import stripe

from appconf import AppConf


def load_path_attr(path):
    i = path.rfind(".")
    module, attr = path[:i], path[i + 1:]
    try:
        mod = importlib.import_module(module)
    except ImportError as e:
        raise ImproperlyConfigured(
            "Error importing {0}: '{1}'".format(module, e)
        )
    try:
        attr = getattr(mod, attr)
    except AttributeError:
        raise ImproperlyConfigured(
            "Module '{0}' does not define a '{1}'".format(module, attr)
        )
    return attr


class PinaxStripeAppConf(AppConf):

    PUBLIC_KEY = None
    SECRET_KEY = None
    API_VERSION = "2015-10-16"
    INVOICE_FROM_EMAIL = "billing@example.com"
    DEFAULT_PLAN = None
    HOOKSET = "pinax.stripe.hooks.DefaultHookSet"
    SEND_EMAIL_RECEIPTS = True
    SUBSCRIPTION_REQUIRED_EXCEPTION_URLS = []
    SUBSCRIPTION_REQUIRED_REDIRECT = None

    class Meta:
        prefix = "pinax_stripe"
        required = ["PUBLIC_KEY", "SECRET_KEY", "API_VERSION"]

    def configure_api_version(self, value):
        stripe.api_version = value
        return value

    def configure_secret_key(self, value):
        stripe.api_key = value
        return value

    def configure_hookset(self, value):
        return load_path_attr(value)()



from django.core.urlresolvers import reverse
from django.test import TestCase
from django.utils import timezone

from django.contrib.auth import authenticate, login, logout, get_user_model

from mock import Mock

from ..conf import settings
from ..middleware import ActiveSubscriptionMiddleware
from ..models import Customer, Subscription, Plan


class DummySession(dict):

    def cycle_key(self):
        return

    def flush(self):
        return


class ActiveSubscriptionMiddlewareTests(TestCase):
    urls = 'pinax.stripe.tests.test_urls'

    def setUp(self):
        self.middleware = ActiveSubscriptionMiddleware()
        self.request = Mock()
        self.request.META = {}
        self.request.session = DummySession()

        self.old_urls = settings.PINAX_STRIPE_SUBSCRIPTION_REQUIRED_EXCEPTION_URLS
        settings.PINAX_STRIPE_SUBSCRIPTION_REQUIRED_EXCEPTION_URLS += (
            'signup',
            'password_reset'
        )

        user = get_user_model().objects.create_user(username="patrick")
        user.set_password("eldarion")
        user.save()
        user = authenticate(username="patrick", password="eldarion")
        login(self.request, user)

    def tearDown(self):
        settings.PINAX_STRIPE_SUBSCRIPTION_REQUIRED_EXCEPTION_URLS = self.old_urls

    def test_authed_user_with_no_customer_redirects_on_non_exempt_url(self):
        self.request.path = "/the/app/"
        response = self.middleware.process_request(self.request)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(
            response._headers["location"][1],
            reverse(settings.PINAX_STRIPE_SUBSCRIPTION_REQUIRED_REDIRECT)
        )

    def test_authed_user_with_no_customer_passes_with_exempt_url(self):
        self.request.path = "/accounts/signup/"
        response = self.middleware.process_request(self.request)
        self.assertIsNone(response)

    def test_authed_user_with_no_customer_passes_with_exempt_url_containing_pattern(self):
        self.request.path = "/password/reset/confirm/test-token/"
        response = self.middleware.process_request(self.request)
        self.assertIsNone(response)

    def test_authed_user_with_no_active_subscription_passes_with_exempt_url(self):
        Customer.objects.create(stripe_id="cus_1", user=self.request.user)
        self.request.path = "/accounts/signup/"
        response = self.middleware.process_request(self.request)
        self.assertIsNone(response)

    def test_authed_user_with_no_active_subscription_redirects_on_non_exempt_url(self):
        Customer.objects.create(stripe_id="cus_1", user=self.request.user)
        self.request.path = "/the/app/"
        response = self.middleware.process_request(self.request)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(
            response._headers["location"][1],
            reverse(settings.PINAX_STRIPE_SUBSCRIPTION_REQUIRED_REDIRECT)
        )

    def test_authed_user_with_active_subscription_redirects_on_non_exempt_url(self):
        customer = Customer.objects.create(
            stripe_id="cus_1",
            user=self.request.user
        )
        plan = Plan.objects.create(
            amount=10,
            currency="usd",
            interval="monthly",
            interval_count=1,
            name="Pro"
        )
        Subscription.objects.create(
            customer=customer,
            plan=plan,
            quantity=1,
            start=timezone.now(),
            status="active",
            cancel_at_period_end=False
        )
        self.request.path = "/the/app/"
        response = self.middleware.process_request(self.request)
        self.assertIsNone(response)

    def test_unauthed_user_passes(self):
        logout(self.request)
        self.request.path = "/the/app/"
        response = self.middleware.process_request(self.request)
        self.assertIsNone(response)

    def test_staff_user_passes(self):
        self.request.user.is_staff = True
        self.request.path = "/the/app/"
        response = self.middleware.process_request(self.request)
        self.assertIsNone(response)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pinax_stripe', '0002_auto_20151205_1451'),
    ]

    operations = [
        migrations.AlterField(
            model_name='card',
            name='cvc_check',
            field=models.CharField(blank=True, max_length=15),
        ),
    ]



import datetime
import decimal

from django.test import TestCase
from django.utils import timezone

from django.contrib.auth import get_user_model

from ..models import Customer, Subscription, Charge, Plan


class CustomerManagerTest(TestCase):

    def setUp(self):
        User = get_user_model()
        # create customers and current subscription records
        period_start = datetime.datetime(2013, 4, 1, tzinfo=timezone.utc)
        period_end = datetime.datetime(2013, 4, 30, tzinfo=timezone.utc)
        start = datetime.datetime(2013, 1, 1, tzinfo=timezone.utc)
        self.plan = Plan.objects.create(
            stripe_id="p1",
            amount=10,
            currency="usd",
            interval="monthly",
            interval_count=1,
            name="Pro"
        )
        self.plan2 = Plan.objects.create(
            stripe_id="p2",
            amount=5,
            currency="usd",
            interval="monthly",
            interval_count=1,
            name="Light"
        )
        for i in range(10):
            customer = Customer.objects.create(
                user=User.objects.create_user(username="patrick{0}".format(i)),
                stripe_id="cus_xxxxxxxxxxxxxx{0}".format(i)
            )
            Subscription.objects.create(
                stripe_id="sub_{}".format(i),
                customer=customer,
                plan=self.plan,
                current_period_start=period_start,
                current_period_end=period_end,
                status="active",
                start=start,
                quantity=1
            )
        customer = Customer.objects.create(
            user=User.objects.create_user(username="patrick{0}".format(11)),
            stripe_id="cus_xxxxxxxxxxxxxx{0}".format(11)
        )
        Subscription.objects.create(
            stripe_id="sub_{}".format(11),
            customer=customer,
            plan=self.plan,
            current_period_start=period_start,
            current_period_end=period_end,
            status="canceled",
            canceled_at=period_end,
            start=start,
            quantity=1
        )
        customer = Customer.objects.create(
            user=User.objects.create_user(username="patrick{0}".format(12)),
            stripe_id="cus_xxxxxxxxxxxxxx{0}".format(12)
        )
        Subscription.objects.create(
            stripe_id="sub_{}".format(12),
            customer=customer,
            plan=self.plan2,
            current_period_start=period_start,
            current_period_end=period_end,
            status="active",
            start=start,
            quantity=1
        )

    def test_started_during_no_records(self):
        self.assertEquals(
            Customer.objects.started_during(2013, 4).count(),
            0
        )

    def test_started_during_has_records(self):
        self.assertEquals(
            Customer.objects.started_during(2013, 1).count(),
            12
        )

    def test_canceled_during(self):
        self.assertEquals(
            Customer.objects.canceled_during(2013, 4).count(),
            1
        )

    def test_canceled_all(self):
        self.assertEquals(
            Customer.objects.canceled().count(),
            1
        )

    def test_active_all(self):
        self.assertEquals(
            Customer.objects.active().count(),
            11
        )

    def test_started_plan_summary(self):
        for plan in Customer.objects.started_plan_summary_for(2013, 1):
            if plan["subscription__plan"] == self.plan:
                self.assertEquals(plan["count"], 11)
            if plan["subscription__plan"] == self.plan2:
                self.assertEquals(plan["count"], 1)

    def test_active_plan_summary(self):
        for plan in Customer.objects.active_plan_summary():
            if plan["subscription__plan"] == self.plan:
                self.assertEquals(plan["count"], 10)
            if plan["subscription__plan"] == self.plan2:
                self.assertEquals(plan["count"], 1)

    def test_canceled_plan_summary(self):
        for plan in Customer.objects.canceled_plan_summary_for(2013, 1):
            if plan["subscription__plan"] == self.plan:
                self.assertEquals(plan["count"], 1)
            if plan["subscription__plan"] == self.plan2:
                self.assertEquals(plan["count"], 0)

    def test_churn(self):
        self.assertEquals(
            Customer.objects.churn(),
            decimal.Decimal("1") / decimal.Decimal("11")
        )


class ChargeManagerTests(TestCase):

    def setUp(self):
        customer = Customer.objects.create(
            user=get_user_model().objects.create_user(username="patrick"),
            stripe_id="cus_xxxxxxxxxxxxxx"
        )
        Charge.objects.create(
            stripe_id="ch_1",
            customer=customer,
            charge_created=datetime.datetime(2013, 1, 1, tzinfo=timezone.utc),
            paid=True,
            amount=decimal.Decimal("100"),
            amount_refunded=decimal.Decimal("0")
        )
        Charge.objects.create(
            stripe_id="ch_2",
            customer=customer,
            charge_created=datetime.datetime(2013, 1, 1, tzinfo=timezone.utc),
            paid=True,
            amount=decimal.Decimal("100"),
            amount_refunded=decimal.Decimal("10")
        )
        Charge.objects.create(
            stripe_id="ch_3",
            customer=customer,
            charge_created=datetime.datetime(2013, 1, 1, tzinfo=timezone.utc),
            paid=False,
            amount=decimal.Decimal("100"),
            amount_refunded=decimal.Decimal("0")
        )
        Charge.objects.create(
            stripe_id="ch_4",
            customer=customer,
            charge_created=datetime.datetime(2013, 4, 1, tzinfo=timezone.utc),
            paid=True,
            amount=decimal.Decimal("500"),
            amount_refunded=decimal.Decimal("15.42")
        )

    def test_charges_during(self):
        charges = Charge.objects.during(2013, 1)
        self.assertEqual(charges.count(), 3)

    def test_paid_totals_for_jan(self):
        totals = Charge.objects.paid_totals_for(2013, 1)
        self.assertEqual(totals["total_amount"], decimal.Decimal("200"))
        self.assertEqual(totals["total_refunded"], decimal.Decimal("10"))

    def test_paid_totals_for_apr(self):
        totals = Charge.objects.paid_totals_for(2013, 4)
        self.assertEqual(totals["total_amount"], decimal.Decimal("500"))
        self.assertEqual(totals["total_refunded"], decimal.Decimal("15.42"))

    def test_paid_totals_for_dec(self):
        totals = Charge.objects.paid_totals_for(2013, 12)
        self.assertEqual(totals["total_amount"], None)
        self.assertEqual(totals["total_refunded"], None)



from setuptools import find_packages, setup


NAME = "pinax-stripe"
DESCRIPTION = "a payments Django app for Stripe"
AUTHOR = "Pinax Team"
AUTHOR_EMAIL = "team@pinaxproject.com"
URL = "https://github.com/pinax/pinax-stripe"
LONG_DESCRIPTION = """
============
Pinax Stripe
============

.. image:: http://slack.pinaxproject.com/badge.svg
    :target: http://slack.pinaxproject.com/

.. image:: https://img.shields.io/travis/pinax/pinax-stripe.svg
    :target: https://travis-ci.org/pinax/pinax-stripe

.. image:: https://img.shields.io/coveralls/pinax/pinax-stripe.svg
    :target: https://coveralls.io/r/pinax/pinax-stripe

.. image:: https://img.shields.io/pypi/dm/pinax-stripe.svg
    :target:  https://pypi.python.org/pypi/pinax-stripe/

.. image:: https://img.shields.io/pypi/v/pinax-stripe.svg
    :target:  https://pypi.python.org/pypi/pinax-stripe/

.. image:: https://img.shields.io/badge/license-MIT-blue.svg
    :target:  https://pypi.python.org/pypi/pinax-stripe/


This app was formerly called ``django-stripe-payments`` and has been renamed to
avoid namespace collisions and to have more consistancy with Pinax.

Pinax
------

Pinax is an open-source platform built on the Django Web Framework. It is an
ecosystem of reusable Django apps, themes, and starter project templates.
This collection can be found at http://pinaxproject.com.

This app was developed as part of the Pinax ecosystem but is just a Django app
and can be used independently of other Pinax apps.


pinax-stripe
------------

``pinax-stripe`` is a payments Django app for Stripe.

This app allows you to process one off charges as well as signup users for
recurring subscriptions managed by Stripe.
"""

setup(
    name=NAME,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    version="3.2.1",
    license="MIT",
    url=URL,
    packages=find_packages(),
    package_data={
        "pinax.stripe": [
            "templates/pinax/stripe/email/body_base.txt",
            "templates/pinax/stripe/email/body.txt",
            "templates/pinax/stripe/email/subject.txt"
        ]
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
        "Framework :: Django",
    ],
    install_requires=[
        "django-appconf>=1.0.1",
        "jsonfield>=1.0.3",
        "stripe>=1.7.9",
        "django>=1.7",
        "pytz",
        "six",
    ],
    test_suite="runtests.runtests",
    tests_require=[
        "mock",
        "django_forms_bootstrap",
    ],
    zip_safe=False,
)



from django.utils import timezone
from django.utils.encoding import smart_str

import stripe

from . import invoices
from . import sources
from . import subscriptions
from ..conf import settings
from .. import hooks
from .. import models
from .. import utils


def can_charge(customer):
    """
    Can the given customer create a charge

    Args:
        customer: a pinax.stripe.models.Customer object
    """
    if customer.date_purged is not None:
        return False
    if customer.default_source:
        return True
    return False


def create(user, card=None, plan=settings.PINAX_STRIPE_DEFAULT_PLAN, charge_immediately=True):
    """
    Creates a Stripe customer

    Args:
        user: a user object
        card: optionally, the token for a new card
        plan: a plan to subscribe the user to
        charge_immediately: whether or not the user should be immediately
                            charged for the subscription

    Returns:
        the pinax.stripe.models.Customer object that was created
    """
    trial_end = hooks.hookset.trial_period(user, plan)

    stripe_customer = stripe.Customer.create(
        email=user.email,
        source=card,
        plan=plan,
        trial_end=trial_end
    )
    cus = models.Customer.objects.create(
        user=user,
        stripe_id=stripe_customer["id"]
    )
    sync_customer(cus, stripe_customer)

    if plan and charge_immediately:
        invoices.create_and_pay(cus)
    return cus


def get_customer_for_user(user):
    """
    Get a customer object for a given user

    Args:
        user: a user object

    Returns:
        a pinax.stripe.models.Customer object
    """
    return next(iter(models.Customer.objects.filter(user=user)), None)


def purge(customer):
    """
    Deletes the Stripe customer data and purges the linking of the transaction
    data to the Django user.

    Args:
        customer: the pinax.stripe.models.Customer object to purge
    """
    try:
        customer.stripe_customer.delete()
    except stripe.InvalidRequestError as e:
        if not smart_str(e).startswith("No such customer:"):
            # The exception was thrown because the customer was already
            # deleted on the stripe side, ignore the exception
            raise
    customer.user = None
    customer.date_purged = timezone.now()
    customer.save()


def link_customer(event):
    """
    Links a customer referenced in a webhook event message to the event object

    Args:
        event: the pinax.stripe.models.Event object to link
    """
    cus_id = None
    customer_crud_events = [
        "customer.created",
        "customer.updated",
        "customer.deleted"
    ]
    if event.kind in customer_crud_events:
        cus_id = event.message["data"]["object"]["id"]
    else:
        cus_id = event.message["data"]["object"].get("customer", None)

    if cus_id is not None:
        customer = next(iter(models.Customer.objects.filter(stripe_id=cus_id)), None)
        if customer is not None:
            event.customer = customer
            event.save()


def set_default_source(customer, source):
    """
    Sets the default payment source for a customer

    Args:
        customer: a Customer object
        source: the Stripe ID of the payment source
    """
    stripe_customer = customer.stripe_customer
    stripe_customer.default_source = source
    cu = stripe_customer.save()
    sync_customer(customer, cu=cu)


def sync_customer(customer, cu=None):
    """
    Syncronizes a local Customer object with details from the Stripe API

    Args:
        customer: a Customer object
        cu: optionally, data from the Stripe API representing the customer
    """
    if cu is None:
        cu = customer.stripe_customer
    customer.account_balance = utils.convert_amount_for_db(cu["account_balance"], cu["currency"])
    customer.currency = cu["currency"] or ""
    customer.delinquent = cu["delinquent"]
    customer.default_source = cu["default_source"] or ""
    customer.save()
    for source in cu["sources"]["data"]:
        sources.sync_payment_source_from_stripe_data(customer, source)
    for subscription in cu["subscriptions"]["data"]:
        subscriptions.sync_subscription_from_stripe_data(customer, subscription)



from .. import models
from .. import utils


def create_card(customer, token):
    """
    Creates a new card for a customer

    Args:
        customer: the customer to create the card for
        token: the token created from Stripe.js
    """
    source = customer.stripe_customer.sources.create(source=token)
    return sync_payment_source_from_stripe_data(customer, source)


def delete_card(customer, source):
    """
    Deletes a card from a customer

    Args:
        customer: the customer to delete the card from
        source: the Stripe ID of the payment source to delete
    """
    customer.stripe_customer.sources.retrieve(source).delete()
    return delete_card_object(source)


def delete_card_object(source):
    """
    Deletes the local card object (Card)

    Args:
        source: the Stripe ID of the card
    """
    if source.startswith("card_"):
        return models.Card.objects.filter(stripe_id=source).delete()


def sync_card(customer, source):
    """
    Syncronizes the data for a card locally for a given customer

    Args:
        customer: the customer to create or update a card for
        source: data reprenting the card from the Stripe API
    """
    defaults = dict(
        customer=customer,
        name=source["name"] or "",
        address_line_1=source["address_line1"] or "",
        address_line_1_check=source["address_line1_check"] or "",
        address_line_2=source["address_line2"] or "",
        address_city=source["address_city"] or "",
        address_state=source["address_state"] or "",
        address_country=source["address_country"] or "",
        address_zip=source["address_zip"] or "",
        address_zip_check=source["address_zip_check"] or "",
        brand=source["brand"],
        country=source["country"],
        cvc_check=source["cvc_check"] or "",
        dynamic_last4=source["dynamic_last4"] or "",
        exp_month=source["exp_month"],
        exp_year=source["exp_year"],
        funding=source["funding"] or "",
        last4=source["last4"] or "",
        fingerprint=source["fingerprint"] or ""
    )
    card, created = models.Card.objects.get_or_create(
        stripe_id=source["id"],
        defaults=defaults
    )
    return utils.update_with_defaults(card, defaults, created)


def sync_bitcoin(customer, source):
    """
    Syncronizes the data for a Bitcoin receiver locally for a given customer

    Args:
        customer: the customer to create or update a Bitcoin receiver for
        source: data reprenting the Bitcoin receiver from the Stripe API
    """
    defaults = dict(
        customer=customer,
        active=source["active"],
        amount=utils.convert_amount_for_db(source["amount"], source["currency"]),
        amount_received=utils.convert_amount_for_db(source["amount_received"], source["currency"]),
        bitcoin_amount=source["bitcoin_amount"],
        bitcoin_amount_received=source["bitcoin_amount_received"],
        bitcoin_uri=source["bitcoin_uri"],
        currency=source["currency"],
        description=source["description"],
        email=source["email"],
        filled=source["filled"],
        inbound_address=source["inbound_address"],
        payment=source["payment"] if "payment" in source else "",
        refund_address=source["refund_address"] or "",
        uncaptured_funds=source["uncaptured_funds"],
        used_for_payment=source["used_for_payment"]
    )
    receiver, created = models.BitcoinReceiver.objects.get_or_create(
        stripe_id=source["id"],
        defaults=defaults
    )
    return utils.update_with_defaults(receiver, defaults, created)


def sync_payment_source_from_stripe_data(customer, source):
    """
    Syncronizes the data for a payment source locally for a given customer

    Args:
        customer: the customer to create or update a Bitcoin receiver for
        source: data reprenting the payment source from the Stripe API
    """
    if source["id"].startswith("card_"):
        return sync_card(customer, source)
    else:
        return sync_bitcoin(customer, source)


def update_card(customer, source, name=None, exp_month=None, exp_year=None):
    """
    Updates a card for a given customer

    Args:
        customer: the customer for whom to update the card
        source: the Stripe ID of the card to update
        name: optionally, a name to give the card
        exp_month: optionally, the expiration month for the card
        exp_year: optionally, the expiration year for the card
    """
    stripe_source = customer.stripe_customer.sources.retrieve(source)
    if name is not None:
        stripe_source.name = name
    if exp_month is not None:
        stripe_source.exp_month = exp_month
    if exp_year is not None:
        stripe_source.exp_year = exp_year
    s = stripe_source.save()
    return sync_payment_source_from_stripe_data(customer, s)



import sys
import traceback

from .. import models


def log_exception(data, exception, event=None):
    """
    Log an exception that was captured as a result of processing events

    Args:
        data: the data to log about the exception
        exception: the exception object itself
        event: optionally, the event object from which the exception occurred
    """
    info = sys.exc_info()
    info_formatted = "".join(traceback.format_exception(*info)) if info[1] is not None else ""
    models.EventProcessingException.objects.create(
        event=event,
        data=data or "",
        message=str(exception),
        traceback=info_formatted
    )



from django.core.management.base import BaseCommand

from django.contrib.auth import get_user_model

from ...actions import customers, charges, invoices


class Command(BaseCommand):

    help = "Sync customer data"

    def handle(self, *args, **options):
        User = get_user_model()
        qs = User.objects.exclude(customer__isnull=True)
        count = 0
        total = qs.count()
        for user in qs:
            count += 1
            perc = int(round(100 * (float(count) / float(total))))
            username = getattr(user, user.USERNAME_FIELD)
            print("[{0}/{1} {2}%] Syncing {3} [{4}]".format(
                count, total, perc, username, user.pk
            ))
            customer = customers.get_customer_for_user(user)
            customers.sync_customer(customer)
            invoices.sync_invoices_for_customer(customer)
            charges.sync_charges_for_customer(customer)
